@@ -1,6 +1,5 @@
 estfun.lmerMod <- function(object, level = 2, ...) {
   if (!is(object, "lmerMod")) stop("estfun.lmerMod() only works for lmer() models.")
-  if (object@devcomp$dims[10] != 0) stop("estfun.lmerMod() only works for ML estimation.")
     
   ## get all elements by getME and exclude multiple random effect models.
   parts <- getME(object, "ALL")
@@ -13,6 +12,11 @@ estfun.lmerMod <- function(object, level = 2, ...) {
   M <- solve(chol(V))
   invV <- tcrossprod(M, M)
   yXbesoV <- crossprod(yXbe, invV)
+  LambdaInd <- parts$Lambda
+  LambdaInd@x[] <- as.double(parts$Lind)
+  invVX <- crossprod(parts$X, invV)
+  Pmid <- solve(crossprod(parts$X, t(invVX)))
+  P <- invV - tcrossprod(crossprod(invVX, Pmid), t(invVX))
   
   ## adapt from Stroup book page 131, last eq,
   ## score for fixed effects parameter: score_beta=XR^{-1}(y-Zb-X\beta)
@@ -26,29 +30,40 @@ estfun.lmerMod <- function(object, level = 2, ...) {
   devV <- vector ("list", (uluti + 1))
   
   for (i in 1:uluti) {
-    devLambda[[i]] <- parts$Lambda
-    devLambda[[i]][which(devLambda[[i]] != parts$theta[i])] <- 0
-    devLambda[[i]][which(parts$Lambda == parts$theta[i])] <- 1
-    devLambda[[i]] <- forceSymmetric(devLambda[[i]], uplo = "L")
+    devLambda[[i]] <- forceSymmetric(LambdaInd==i, uplo = "L")
     devV[[i]] <- tcrossprod(tcrossprod(parts$Z, t(devLambda[[i]])), parts$Z)
   }
   devV[[(uluti+1)]] <- diag(1,nrow(parts$X))
   
   ## score for variance covariance parameter
   score_varcov <- matrix(NA, nrow = nrow(parts$X), ncol = (uluti + 1))
-  for (j in 1:length(devV)) {
-    score_varcov[,j] <- as.vector(-(1/2) * diag(crossprod(invV, devV[[j]])) +
+  
+  ## ML estimates. 
+  if (object@devcomp$dims[10] == 0) {
+    for (j in 1:length(devV)) {
+      score_varcov[,j] <- as.vector(-(1/2) * diag(crossprod(invV, devV[[j]])) +
       t((1/2) * tcrossprod(tcrossprod(yXbesoV, t(devV[[j]])), invV)) *
       (yXbe))
+    }
+  }
+
+  ## REML estimates
+   if (object@devcomp$dims[10] == 2|object@devcomp$dims[10] == 1) {
+    for (j in 1:length(devV)) {
+      score_varcov[,j] <- as.vector(-(1/2) * diag(crossprod(P, devV[[j]])) +
+      t((1/2) * tcrossprod(tcrossprod(yXbesoV, t(devV[[j]])), invV)) *
+      (yXbe))
+    }
   }
   
   ## Organize score matrix
   score <- cbind(as.matrix(score_beta), score_varcov)
   colnames(score) <- c(names(parts$fixef),
-    paste("cov", names(parts$theta), sep="_"), "residual")
+                       paste("cov", names(parts$theta), sep="_"), "residual")
+  
   ## Clusterwise scores if level==2
   if (level == 2) {
-    index <- rep(1:parts$l_i, (parts$n/parts$l_i))
+    index <- parts$flist[[1]]
     index <- index[order(index)]
     score <- aggregate(x = score, by = list(index), FUN = sum)[,-1]
   }
